#version 420

layout(points) in;
layout (triangle_strip, max_vertices=4) out;
 
out vec2 uv;
out vec3 vposition;

uniform mat4 viewProjectionMatrix;
uniform mat4 modelMatrix;
uniform vec3 gCameraPos;
uniform vec2 scale;
uniform float waterHeight;


void insertVertex(vec3 position, vec2 _uv)
{
	uv = _uv;
	gl_Position = viewProjectionMatrix * vec4(position, 1.0);
	EmitVertex();
}

void main()
{
	if (gl_in[0].gl_Position.y >= waterHeight) // TODO: discard everything below the waterplane in the vertex shader with clipping plane and not in the fragment shader
	{		
		vec3 Pos = gl_in[0].gl_Position.xyz;	
		vec3 toCamera = normalize(gCameraPos - Pos);
		vec3 up = vec3(0.0f, 1.0f, 0.0f);
		vec3 scaledRight = normalize(cross(up, toCamera)) * scale.x;
		vec3 scaledUp = up * scale.y;
	    
		Pos -= (scaledUp * 0.5);
		Pos -= (scaledRight * 0.5);
		insertVertex(Pos, vec2(0.0f, 0.0f)); //v0

		Pos += scaledRight;
		insertVertex(Pos, vec2(1.0f, 0.0f)); //v1
		Pos -= scaledRight;
		Pos += scaledUp;		
		insertVertex(Pos, vec2(0.0f, 1.0f)); //v2
    
		Pos += scaledRight;
		insertVertex(Pos, vec2(1.0, 1.0));	//v3
		
		EndPrimitive();
	}
}
