#version 420

layout(points) in;
layout (triangle_strip, max_vertices=24) out;
 
in VertexData {
	vec3 position;
	mat4 rotationMatrix;
	float wingposition;
	vec3 corridordirection;
} VertexIn[];

out	vec3 position;
out vec3 normal;
out vec2 uv;
flat out vec3 corridordirection;
flat out unsigned int tex;

uniform unsigned int matrixMode;
uniform mat4 vpMatrix;
uniform float t;
uniform vec3 gCameraPos;
uniform float waterHeight;
uniform unsigned int birdMode;
uniform unsigned int shadowMode;

void insertVertex(vec3 offset, vec2 _uv, unsigned int _tex)
{
	float scalefactor = 0.1f;
	vec4 pos;
	if (_tex == 1) //fire quad billboard
	{
		//just take precalculated billboard position
		pos = vec4(offset, 1.0f);
		uv = _uv;
	}
	else //the bird itself
	{
		//first scale, then rotate then translate
		pos = gl_in[0].gl_Position + VertexIn[0].rotationMatrix * (vec4(offset,0.0f) * scalefactor);
		uv = _uv + VertexIn[0].corridordirection.z * vec2(corridordirection.x,corridordirection.y);
	}

	position = vec3(pos);
	normal = vec3(1.0f,1.0f,1.0f);
	tex = _tex;
	corridordirection = VertexIn[0].corridordirection;
	gl_Position = vpMatrix * pos;
	EmitVertex();
}

void insertFireBillBoard(vec4 sourcePos)
{
	float scalefactor = 0.6f;
	float scalex = 0.5f * scalefactor;
	float scaley = 1.0f * scalefactor;
	vec3 Pos = sourcePos.xyz;	
	vec3 toCamera = normalize(gCameraPos - Pos);
    vec3 up = vec3(0.0f, 1.0f, 0.0f);
    vec3 scaledRight = normalize(cross(up, toCamera)) * scalex;
	vec3 scaledUp = up * scaley;
	    
	Pos -= (scaledRight * 0.5);
    insertVertex(Pos, vec2(0.0f, 0.0f), 1); //v0

    Pos += scaledRight;
    insertVertex(Pos, vec2(1.0f, 0.0f), 1); //v1
    Pos -= scaledRight;
	Pos += scaledUp;		
    insertVertex(Pos, vec2(0.0f, 1.0f), 1); //v2
    
	Pos += scaledRight;
    insertVertex(Pos, vec2(1.0, 1.0), 1);	//v3
	EndPrimitive();
}

void main()
{
	if (VertexIn[0].position.y >= waterHeight) // TODO: discard everything below the waterplane in the vertex shader with clipping plane and not in the fragment shader
	{
		float scalefactor = 0.1f;	
		insertVertex(vec3(0.0f,0.0f,2.0f), vec2(0.5f,0.0f), 0); //v0
		insertVertex(vec3(1.0f,0.0f,1.0f), vec2(0.67f,0.25f), 0); //v1
		insertVertex(vec3(-1.0f,0.0f,1.0f), vec2(0.33f,0.25f), 0); //v2
		insertVertex(vec3(1.0f,0.0f,-1.0f), vec2(0.67f,0.75f), 0); //v3
		insertVertex(vec3(-1.0f,0.0f,-1.0f), vec2(0.33f,0.75f), 0); //v4
		insertVertex(vec3(0.0f,0.0f,-2.0f), vec2(0.5f,1.0f), 0); //v5
		EndPrimitive();
		insertVertex(vec3(-1.0f,0.0f,1.0f), vec2(0.33f,0.25f), 0); //v2
		insertVertex(vec3(-1.0f,0.0f,-1.0f), vec2(0.33f,0.75f), 0); //v4
		insertVertex(vec3(-3.0f,VertexIn[0].wingposition*2.0f,0.0f), vec2(0.0f,0.5f), 0); //v6 //leftwing
		EndPrimitive();
		insertVertex(vec3(1.0f,0.0f,1.0f), vec2(0.67f,0.25f), 0); //v1
		insertVertex(vec3(3.0f,VertexIn[0].wingposition*2.0f,0.0f), vec2(1.0f,0.5f), 0); //v7 //rightwing
		insertVertex(vec3(1.0f,0.0f,-1.0f), vec2(0.67f,0.75f), 0); //v3
		EndPrimitive();
	
		//fire
		if (matrixMode == 0 && birdMode == 1 && shadowMode != 1) //firebird
		{
			insertFireBillBoard(gl_in[0].gl_Position + VertexIn[0].rotationMatrix * (vec4(0.001f, 0.0f, 0.001f, 1.0f) * scalefactor)); //middle one
			//insertFireBillBoard(gl_in[0].gl_Position + VertexIn[0].rotationMatrix * (vec4(1.5f,0.0f,0.0f, 1.0f) * scalefactor)); //right one
			//insertFireBillBoard(gl_in[0].gl_Position + VertexIn[0].rotationMatrix * (vec4(-1.5f,0.0f,0.0f, 1.0f) * scalefactor)); //left one
		}
	}
}
