#version 420
   
uniform sampler2D SplineTexture; //the texture with the sampled curve values
uniform float timeDiff; //time since last frame
uniform float totalTime; //time the bird needs for one total walkthrough || default: 50
uniform float totalCurveParts; // defines, how often in the curve the bird rearranges with new direction || default: 100
uniform float windingFactor; //defines, how often the bird winds around the spline for one total walkthrough.  || default: 15
uniform float wingSpeed; //defines, how often a bird flaps his wings multiplied with current speed || default: 2
uniform unsigned int hullMode; //if true, all birds fly on the outer hull of the corridor
uniform unsigned int birdMode;

uniform unsigned int run;
uniform float totalTimeElapsed;

layout (location=0) in vec4 position0;
layout (location=1) in vec3 direction0;
layout (location=2) in float timecurrent0;
layout (location=3) in float speed0;
layout (location=4) in float wingposition0;
layout (location=5) in float curvepart0;
layout (location=6) in vec3 corridordirection0;

out vec4 position1;
out vec3 direction1;
out float timecurrent1;
out float speed1;
out float wingposition1;
out float curvepart1;
out vec3 corridordirection1;

vec3 rotateArbitrary(vec3 p,float theta,vec3 r)
{
   vec3 q = vec3(0.0f,0.0f,0.0f);
   float costheta,sintheta;

   r = normalize(r);
   costheta = cos(theta);
   sintheta = sin(theta);

   q.x += (costheta + (1 - costheta) * r.x * r.x) * p.x;
   q.x += ((1 - costheta) * r.x * r.y - r.z * sintheta) * p.y;
   q.x += ((1 - costheta) * r.x * r.z + r.y * sintheta) * p.z;

   q.y += ((1 - costheta) * r.x * r.y + r.z * sintheta) * p.x;
   q.y += (costheta + (1 - costheta) * r.y * r.y) * p.y;
   q.y += ((1 - costheta) * r.y * r.z - r.x * sintheta) * p.z;

   q.z += ((1 - costheta) * r.x * r.z - r.y * sintheta) * p.x;
   q.z += ((1 - costheta) * r.y * r.z + r.x * sintheta) * p.y;
   q.z += (costheta + (1 - costheta) * r.z * r.z) * p.z;
   return(q);
}

float rand(vec2 co){ return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453); }

void main()
{
	position1 = position0;
	direction1 = direction0;
	timecurrent1 = timecurrent0;
	speed1 = speed0;
	wingposition1 = wingposition0;
	curvepart1 = curvepart0;
	corridordirection1 = corridordirection0;

	if (run == 1 || totalTimeElapsed < 2.0f)
	{
		timecurrent1 = timecurrent0 + timeDiff;
		//float totaltime = 50;//total time, a bird should need for the curve
		//float totalcurveparts = 100; //totalcurveparts parts of the curve
	
		if (timecurrent0 < 0.0f) //time to start not yet reached, wait for start
		{
			position1.xyz = texture2D(SplineTexture, vec2(0.001f,0.0f)).xyz;
			position1.w = 0.001f;
			//direction1 = vec3(1.0f,0.0f,0.0f);
			if (timecurrent1 >= 0.0f) { timecurrent1 = 0.0f; } //start time reached, set the start time to 0
		}
		else if (timecurrent0 >= totalTime - corridordirection1.z)
		{
			//time is over, start from beginning
			//timecurrent1 = -0.01f;
			//curvepart1 = 0;
			//do nothing, stay at the end
		}
		else //bird is on its way
		{
			if (birdMode == 3) //crazybird, flys in random dir
			{
				//if (corridordirection1.z) < 0.2) { direction1 = normalize(1.0, 0.0, 0.0); }
				direction1 = normalize(corridordirection1);
				speed1 = 40.0;
			}
			else
			{	
				float u = position0.w; //we are at this point

				if (u + 0.01f >= curvepart0/totalCurveParts) //jump into a new curvepart, calculate a new direction
				{
					curvepart1 = curvepart0 + 1.0f;
					float unext = u + 1.0f/totalCurveParts;
					//vec3 thissamplepoint = vec3(texture2D(SplineTexture, vec2(clamp(u,0.001f,0.999f),0.0f)));			//where would i be now, if no corridordirection would be applied, and i would be right on the spline
					//vec3 nextsamplepoint = vec3(texture2D(SplineTexture, vec2(clamp(unext,0.001f,0.999f),0.0f))); //where would i be at the end of the next curvepart, if i would be right on the spline
					//float nextcorridorradius = (texture2D(SplineTexture, vec2(clamp(unext,0.001f,0.999f),1.0f))).x; //next corridorradius is saved in the texture row 3, which is currently the last row, so v-coord is 1.0
					vec3 thissamplepoint = vec3(texture2D(SplineTexture, vec2(clamp(u,0.001f,0.999f),0.0f)));			//where would i be now, if no corridordirection would be applied, and i would be right on the spline
					vec4 nextsamplepointcombined = texture2D(SplineTexture, vec2(clamp(unext,0.001f,0.999f),0.0f));
					vec3 nextsamplepoint = nextsamplepointcombined.xyz; //where would i be at the end of the next curvepart, if i would be right on the spline
					float nextcorridorradius = nextsamplepointcombined.w; //next corridorradius is saved in the w component of the texture
			
					//calculate direction shift
					vec3 splinedirection = normalize(nextsamplepoint.xyz - thissamplepoint.xyz);
					vec3 up = vec3(0.0f,1.0f,0.0f);
					vec3 normalvector = cross(up, splinedirection);
					if (length(normalvector) < 0.001) {  normalvector = vec3(+1.0f,0.0f,0.0f); 	}
					normalvector = normalize(normalvector); 
					float angle = u * 6.283 * windingFactor + corridordirection0.y;
					vec3 resultdirection = rotateArbitrary(/*abs*/(normalvector), angle, splinedirection);
					float corridordirectionradius = corridordirection0.x;
					if (hullMode == 1) { corridordirectionradius = 1.0f; }
					direction1 = nextsamplepoint.xyz - position0.xyz + resultdirection * nextcorridorradius * corridordirectionradius;
					speed1 = length(direction1) / (totalTime/totalCurveParts);
					direction1 = normalize(direction1);
				}
			}
		
			//calculate new position and time

			position1.xyz = position0.xyz + direction1 * timeDiff * speed1;
			position1.w = timecurrent1/totalTime;
		}
	 
		wingposition1 = wingposition0 + timeDiff * speed1 * wingSpeed; //* (rand(position0.xy + vec2(speed0, speed0))+3.7);  
		if (wingposition1 > 11.0f) { wingposition1 = -11.0f; }
		if (wingposition1 > -9.0f && wingposition1 < 0) { wingposition1 = 9.0f; }
	}
}