#version 400

precision highp float;

uniform sampler2D positionTexture;
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D normalTexture;
uniform sampler2D ssaoTexture;
//uniform sampler2D depthTexture;

uniform sampler2D shadowTexture;
uniform mat4 shadowMatrix;
uniform mat4 biasMatrix;

uniform vec3 camPos;
uniform vec3 lightDirection;
uniform unsigned int isSSAOEnabled;
uniform unsigned int isShadowMappingEnabled;
uniform vec3 lightColor;

in vec2 vuv;

out vec4 out_color;

const vec4 black = vec4(0.0f,0.0f,0.0f,0.0f);

void main()
{
	vec4 pos = texture(positionTexture, vuv);
	vec4 color = texture(diffuseTexture, vuv);
	vec4 specular = texture(specularTexture, vuv);
	vec4 normalT = texture(normalTexture, vuv);
	vec3 normal = normalize(normalT.xyz);
    float shadow = 1.0;	

	if (isShadowMappingEnabled == 1)
	{
		vec4 shadowPos = shadowMatrix * vec4(pos.xyz, 1.0);
		float w = shadowPos.w;
		shadowPos = shadowPos / shadowPos.w;
		vec4 shadowPosBiased = biasMatrix * shadowPos;
	
		if (shadowPosBiased.x < 1.0 && shadowPosBiased.y < 1.0  && shadowPosBiased.x > 0.0 && shadowPosBiased.y > 0.0) //inside the shadowmap
		{
			float shadowZ = texture(shadowTexture, shadowPosBiased.xy).x;
			float bias = 0.02; //0.02
			shadowPosBiased.z -= bias;
        
			if (shadowZ < shadowPosBiased.z && shadowPosBiased.z < 0.99 - bias) { shadow = 0.3; }
			//out_color = vec4(shadowZ - shadowPosBiased.z,  bias, shadow, 1.0);	
		}
	}
	 
	vec3 viewRay = normalize(camPos - pos.xyz);
	vec3 lightRay = normalize(-lightDirection);

	vec3 halfVec = normalize((lightRay + viewRay) / 2.0f);

	float NdL = clamp(dot(normal, lightRay),0.0f,1.0f);
	float NdH = clamp(dot(normal, halfVec),0.0f,1.0f);

	vec3 aCol = clamp(pos.a * color.rgb,0.0f,1.0f);
	vec3 dCol = clamp(color.a * NdL * color.rgb,0.0f,1.0f);
	vec3 sCol = clamp(specular.a * pow(NdH,normalT.a) * specular.rgb,0.0f,1.0f);
	
	aCol *= lightColor;
	dCol *= lightColor;
	sCol *= lightColor;

	float ssao = 1.0f;
	if(isSSAOEnabled != 0)
	{
		ssao = 1.0f - texture(ssaoTexture, vuv).r;
	}
	
	out_color = vec4((aCol * ssao + dCol * ssao + sCol) * shadow,1.0); 
	if (pos.a > 4.9f) { out_color = vec4((color.rgb + sCol) * shadow, 1.0); } //TODO: this is a quick hack, see water fragment shader
	
}