#version 420

uniform sampler2D diffuseTexture;
uniform sampler2D alphaTexture;

uniform sampler2D matrixTexture;
uniform sampler2D matrixGlowTexture;
uniform unsigned int matrixMode;
uniform vec2 matrixScale;
uniform vec2 matrixGlowOffset;
uniform float matrixGlowAmount;
uniform float waterHeight;

uniform float ambientCoefficient;
uniform float diffuseCoefficient;
uniform float specularCoefficient;
uniform float specularHardness;
uniform unsigned int shadowMode;

in vec3 teposition;
in vec3 tenormal;
in vec3 teuv;

out vec4 out_color[4];

void main()
{
	if (teposition.y < waterHeight) { discard; } // TODO: discard everything below the waterplane in the vertex shader with clipping plane and not in the fragment shader
	if (shadowMode == 1) 
	{ 	
		out_color[0] = vec4(gl_FragCoord.z,gl_FragCoord.z, gl_FragCoord.z, 1.0); 
	}
	else
	{	
		float alpha = texture(alphaTexture, teuv.xy).r;

		if(alpha > 0.5f)
		{
			vec4 color;
			if(matrixMode == 0)
			{
				color = texture(diffuseTexture, vec2(teuv.y,teuv.x));
			}
			else
			{
				color = texture(matrixTexture, teuv.xy * matrixScale + vec2(teuv.z,0.0f));
			
				float glow = texture(matrixGlowTexture, teuv.xy * matrixScale + matrixGlowOffset + vec2(teuv.z,0.0f)).r;

				if(glow > 0.0f)
				{
					color *= matrixGlowAmount;
				}
			}

			float fac = teuv.z * 0.5f + 0.5f;
			color *= fac;

			out_color[0] = vec4(teposition,ambientCoefficient);
			out_color[1] = vec4(color.rgb,diffuseCoefficient);
			out_color[2] = vec4(color.rgb,specularCoefficient);
			out_color[3] = vec4(tenormal,specularHardness);
		}
		else
		{
			discard;
		}
	}
}