#version 420

uniform vec3 camPos;
uniform float maxDist;
uniform float maxBend;
uniform float time;

uniform vec4 bounds; //x = xMin y = xMax z = zMin w = zMax
uniform sampler2D densityTexture;
uniform sampler2D heightmapTexture;

uniform float heightScale;
uniform float grassHeightFac;
uniform unsigned int intro;

in vec4 position; //w = random number for pass through
in vec4 midPosition; //y = 1.0f if top vertex 0.0f if bot vertex; w = random number
in vec4 uv; //z & w = random number

out vec4 vuv; //z = Distance to camera; w = random number
out float vhere;

void main()
{
	vec2 lookup = midPosition.xz;
	lookup.x = (lookup.x + bounds.x) / (bounds.y - bounds.x);
	lookup.y = (lookup.y + bounds.z) / (bounds.w - bounds.z);

	vec2 density = texture(densityTexture,lookup).xy;

	if(max(density.x,density.y) <= position.w)
	{
		vhere = 0.0f;
	}
	else
	{
		vhere = 1.0f;
		float dist = distance(vec3(camPos.x,0.0f,camPos.z), vec3(midPosition.x,0.0f,midPosition.z));
		vuv = vec4(uv.xy, dist, position.w);

		vec3 pos;
		if(midPosition.y == 1.0f)
		{
			float wind = sin(0.15f * 3.1415f * midPosition.x + time + 0.78539f / (cos(0.1f * 3.1415f * midPosition.z) + 0.0001f)) * cos(0.2f * 3.1415f * midPosition.x + time + 0.78539f / (cos(0.1f * 3.1415f * midPosition.z) + 0.0001f));
			vec3 offset = vec3(maxBend * (uv.z * 2.0f - 1.0f) * wind, 0.0f, maxBend * (uv.w * 2.0f - 1.0f) * wind);
			pos = position.xyz + offset;
			float distFac = 1.0f - (pow(dist,0.8f) / pow(maxDist,0.8f));
			if(intro == 0)
			{
				pos.y *= distFac;
			}
			else
			{
				pos.y *= grassHeightFac * density.x + 1.0f * density.y;
			}
		}
		else
		{
			pos = position.xyz;
		}

		float height = texture(heightmapTexture,lookup).x;
		pos.y += height * heightScale;

		float yAngle = acos(midPosition.w * 2.0f - 1.0f);

		mat4 yMatrix = mat4(cos(yAngle), 0, -sin(yAngle), 0,
							0, 1, 0, 0,
							sin(yAngle), 0, cos(yAngle), 0,
							0, 0, 0, 1);

		mat4 tBackMatrix = mat4(1,0,0,0,
								0,1,0,0,
								0,0,1,0,
								-midPosition.x,0,-midPosition.z,1);

		mat4 tPosMatrix = mat4(1,0,0,0,
							   0,1,0,0,
							   0,0,1,0,
							   midPosition.x,0,midPosition.z,1);

		gl_Position = tPosMatrix * yMatrix * tBackMatrix * vec4(pos,1.0f);
	}
}