#version 400

uniform sampler2D diffuseTexture;

uniform sampler2D matrixTexture;
uniform sampler2D matrixGlowTexture;
uniform unsigned int matrixMode;
uniform float matrixScale;
uniform vec2 matrixGlowOffset;
uniform float matrixGlowAmount;
uniform float waterHeight;

uniform float ambientCoefficient;
uniform float diffuseCoefficient;
uniform float specularCoefficient;
uniform float specularHardness;
uniform unsigned int shadowMode;

in vec3 tepos;
in vec3 tenormal;
in vec4 teuv;

out vec4 out_color[4];

void main()
{
	if (shadowMode == 1) 
	{ 	
		out_color[0] = vec4(gl_FragCoord.z,gl_FragCoord.z, gl_FragCoord.z, 1.0); 
	}
	else
	{	
		if (tepos.y < waterHeight) { discard; } // TODO: discard everything below the waterplane in the vertex shader with clipping plane and not in the fragment shader
	
		vec4 color;
		if(matrixMode == 0)
		{
			color = texture(diffuseTexture, teuv.xy);
		}
		else
		{
			color = texture(matrixTexture, teuv.zw * matrixScale);

			float glow = texture(matrixGlowTexture, teuv.zw * matrixScale + matrixGlowOffset).r;

			if(glow > 0.0f)
			{
				color *= matrixGlowAmount;
			}
		}

		out_color[0] = vec4(tepos,ambientCoefficient);
		out_color[1] = vec4(color.rgb,diffuseCoefficient);
		out_color[2] = vec4(color.rgb,specularCoefficient);
		out_color[3] = vec4(tenormal,specularHardness);
	}
}