#version 400

uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D objectSelectedTexture;
uniform vec3 camPos;

uniform sampler2D matrixTexture;
uniform sampler2D matrixGlowTexture;
uniform unsigned int matrixMode;
uniform float matrixScale;
uniform vec2 matrixGlowOffset;
uniform float matrixGlowAmount;

uniform float ambientCoefficient;
uniform float diffuseCoefficient;
uniform float specularCoefficient;
uniform float specularHardness;
uniform unsigned int isSelected;
uniform unsigned int shadowMode;

in vec3 vposition;
in vec3 vnormal;
in vec2 vuv;

out vec4[4] out_color;

void main()
{
	if (shadowMode == 1) 
	{ 	
		out_color[0] = vec4(gl_FragCoord.z,gl_FragCoord.z, gl_FragCoord.z, 1.0); 
	}
	else
	{
		vec4 diffuseColor;	
		vec4 specularColor;
		if(matrixMode == 0)
		{
			if (isSelected == 1)
			{
				diffuseColor = texture(objectSelectedTexture, vuv);
			}
			else
			{
				diffuseColor = texture(diffuseTexture, vuv);
				specularColor = texture(specularTexture, vuv);
			}
		

			if(specularColor.rgb == vec3(0.0f,0.0f,0.0f))
			{
				specularColor = vec4(1.0f,1.0f,1.0f,1.0f);
			}
		}
		else
		{
			diffuseColor = texture(matrixTexture, vuv * matrixScale);

			float glow = texture(matrixGlowTexture, vuv * matrixScale + matrixGlowOffset).r;

			if(glow > 0.0f)
			{
				diffuseColor *= matrixGlowAmount;
			}

			specularColor = diffuseColor;
		}

		out_color[0] = vec4(vposition,ambientCoefficient);
		out_color[1] = vec4(diffuseColor.rgb,diffuseCoefficient);
		out_color[2] = vec4(specularColor.rgb,specularCoefficient);
		out_color[3] = vec4(vnormal,specularHardness);
	}
}