#version 400

precision highp float;

uniform sampler2D positionTexture;
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D normalTexture;
//uniform sampler2D depthTexture;
uniform vec3 attenuationFactors;
uniform vec3 lightColor;


uniform unsigned int screenWidth;
uniform unsigned int screenHeight;

uniform vec3 cameraPosition;
uniform vec3 lightPosition;

in vec2 vuv;
in vec3 vposition;

out vec4 out_color;

const vec4 black = vec4(0.0f,0.0f,0.0f,0.0f);

void main()
{
	vec2 texCoord = vec2((gl_FragCoord.x +0.0f) / screenWidth,(gl_FragCoord.y +0.0f) / screenHeight);

	vec4 pos = texture(positionTexture, texCoord);
	vec4 color = texture(diffuseTexture, texCoord);
	vec4 specular = texture(specularTexture, texCoord);
	vec4 normalT = texture(normalTexture, texCoord);

	vec3 normal = normalize(normalT.xyz);

	vec3 viewRay = (cameraPosition - pos.xyz);
	vec3 lightRay = (lightPosition - pos.xyz);
	float dist = length(lightRay);
	vec3 aCol = vec3(0.0f,0.0f,0.0f);
	vec3 dCol = vec3(0.0f,0.0f,0.0f);
	vec3 sCol = vec3(0.0f,0.0f,0.0f);

	//if (!(pos.x < 0.0001f && pos.x > -0.0001 && pos.y < 0.0001f && pos.y > -0.0001 && pos.z < 0.0001f && pos.z > -0.0001)) //not skybox
	//{
		viewRay = normalize(viewRay);
		lightRay = normalize(lightRay);

		vec3 halfVec = normalize((lightRay + viewRay) / 2.0f);

		float NdL = clamp(dot(normal, lightRay),0.0f,1.0f);
		float NdH = clamp(dot(normal, halfVec),0.0f,1.0f);

		aCol = clamp(pos.a * color.rgb,0.0f,1.0f);
		dCol = clamp(color.a * NdL * color.rgb,0.0f,1.0f);	
		sCol = clamp(specular.a * pow(NdH,normalT.a) * specular.rgb,0.0f,1.0f);
	//}
	//else //skybox
	//{
	//	dist = 2000.0f + vposition.x; //TODO: 2d screen distance
	//	dCol = color.rgb;
	//}
	
	float att = 1.0f / (attenuationFactors.x + attenuationFactors.y * dist + attenuationFactors.z * dist * dist);

	vec3 lightColorAtt = lightColor * att; 
	aCol *= lightColor * 0.00005;
	dCol *= lightColorAtt;
	sCol *= lightColorAtt;
	//if (specular.a >= 1.0f) //fire
	//{
	//	out_color = vec4(specular.rg * specular.b, 0.0f,1.0f); //veeeery big hack, should not be done (blue channel as alpha information used, because fire has almost no blue component
	//}
	//else
	{
		out_color = vec4(aCol + dCol + sCol,1.0f);
		//out_color = vec4(1.0f,1.0f,1.0f,1.0f);
	}
}