#version 400

uniform sampler2D positionTexture;
uniform sampler2D normalTexture;
uniform sampler2D randomTexture;
uniform sampler2D depthTexture;

uniform vec2 screenSize;
uniform vec2 randomSize;
uniform float sampleRadius;
uniform float intensity;
uniform float scale;
uniform float bias;
uniform float farPlane;
uniform float iterations;

in vec2 vuv;

out float ambientOcclusion;

vec3 getPosition(vec2 uv)
{
	return texture(positionTexture, uv).xyz;
}

vec3 getNormal(vec2 uv)
{
	return normalize(texture(normalTexture, uv).xyz);
}

vec2 getRandom(vec2 uv)
{
	return normalize(texture(randomTexture, screenSize * uv / randomSize).xy * 2.0f - 1.0f);
}

float doAmbientOcclusion(vec2 uv, vec2 ruv, vec3 pos, vec3 norm)
{
	vec3 diff = getPosition(uv + ruv) - pos;
	vec3 v = normalize(diff);
	float d = length(diff) * scale;
	return max(0.0f, dot(norm, v) - bias) * (1.0f / (1.0f + d)) * intensity;
}

void main()
{
	vec3 position = getPosition(vuv);
	vec3 norm = getNormal(vuv);
	vec2 rand = getRandom(vuv);

	float depth = texture(depthTexture, vuv).x * farPlane;

	float ao = 0.0f;
	float rad = sampleRadius / depth;

	for(unsigned int i=0; i<iterations; i++)
	{
		vec2 lookup = reflect(vec2(2 * 3.141592654f / iterations * float(i)), rand) * rad;

		ao += doAmbientOcclusion(vuv, lookup * 0.25f, position, norm);
		ao += doAmbientOcclusion(vuv, lookup, position, norm);
	}

	ambientOcclusion = ao / (iterations * 2.0f);
}