#version 400

uniform sampler2D diffuseTexture;
uniform sampler2D normalTexture;

uniform sampler2D matrixTexture;
uniform sampler2D matrixGlowTexture;
uniform unsigned int matrixMode;
uniform float matrixScale;
uniform vec2 matrixGlowOffset;
uniform float matrixGlowAmount;

uniform float ambientCoefficient;
uniform float diffuseCoefficient;
uniform float specularCoefficient;
uniform float specularHardness;
uniform unsigned int shadowMode;

in vec3 vposition;
in vec3 vnormal;
in vec2 vuv;
in mat3 texSpaceMat;

out vec4 out_color[4];

void main()
{
	if (shadowMode == 1) 
	{ 	
		out_color[0] = vec4(gl_FragCoord.z,gl_FragCoord.z, gl_FragCoord.z, 1.0); 
	}
	else
	{
		vec4 diffuseColor = texture(diffuseTexture,vuv);

		if(diffuseColor.a < 0.8f)
		{
			discard;
		}

		if(matrixMode != 0)
		{
			diffuseColor = texture(matrixTexture, vuv * matrixScale);
			
			float glow = texture(matrixGlowTexture, vuv * matrixScale + matrixGlowOffset).r;

			if(glow > 0.0f)
			{
				diffuseColor *= matrixGlowAmount;
			}
		}

		vec3 normal = texture(normalTexture,vuv).xyz;

		if(normal == vec3(0.0f,0.0f,0.0f))
		{
			normal = vnormal;
		}
		else
		{
			normal = (normal - 0.5f) * 2.0f;
			normal = texSpaceMat * normal;
		}

		out_color[0] = vec4(vposition,ambientCoefficient);
		out_color[1] = vec4(diffuseColor.rgb,diffuseCoefficient);
		out_color[2] = vec4(diffuseColor.rgb,specularCoefficient);
		out_color[3] = vec4(normal,specularHardness);
	}
}