#version 400

uniform sampler2D diffuseTexture;
uniform sampler2D totalTexture; //reflection texture
uniform sampler2D bumpmapTexture;
//uniform sampler2D specularTexture;
//uniform sampler2D objectSelectedTexture;
uniform vec3 camPos;
uniform float timeTotal;

uniform sampler2D matrixTexture;
uniform sampler2D matrixGlowTexture;
uniform unsigned int matrixMode;
uniform float matrixScale;
uniform vec2 matrixGlowOffset;
uniform float matrixGlowAmount;
uniform vec4 lightColor;
uniform vec3 waterColor;

uniform unsigned int screenWidth;
uniform unsigned int screenHeight;

uniform float ambientCoefficient;
uniform float diffuseCoefficient;
uniform float specularCoefficient;
uniform float specularHardness;
uniform unsigned int shadowMode;

in vec3 vposition;
in vec3 vnormal;
in vec2 vuv;
in vec2 vuvBump;

out vec4[4] out_color;

void main()
{
	if (shadowMode == 1) 
	{ 	
		out_color[0] = vec4(gl_FragCoord.z,gl_FragCoord.z, gl_FragCoord.z, 1.0); 
	}
	else
	{	
		vec4 diffuseColor;
		vec4 specularColor;
		vec3 vnormalBumped = vnormal;
		vec4 bumpColor = texture(bumpmapTexture, vuvBump);
		vec2 perturbation = 0.2f * (bumpColor.rg - vec2(0.5f,0.5f));
	
		if(matrixMode == 0)
		{	

			vnormalBumped = normalize(vec3(perturbation.r,1.0f,perturbation.g));
			float correctionHeightFactor = screenHeight / 75.0f;
			diffuseColor = texture(totalTexture, vec2((gl_FragCoord.x +0.0f) / screenWidth,1.0f - ((gl_FragCoord.y - correctionHeightFactor)   / screenHeight)) + perturbation );
			float factor = clamp(length(vposition) * 0.004, 0.0, 1.0);
		
			diffuseColor.rgb = diffuseColor.rgb * (1.0 - factor) + lightColor.rgb * lightColor.a * factor;
			diffuseColor.rgb *= waterColor;
			specularColor = vec4(1.0f, 1.0f, 1.0f, 1.0f);
		
		}
		else
		{
			diffuseColor = texture(matrixTexture, (vuv + perturbation) * matrixScale * 2.0f);

			float glow = texture(matrixGlowTexture, (vuv + perturbation) * matrixScale * 2.0f + matrixGlowOffset).r;

			if(glow > 0.0f)
			{
				diffuseColor *= matrixGlowAmount;
			}

			specularColor = diffuseColor;
		}

		out_color[0] = vec4(vposition,ambientCoefficient + 5.0); //TODO: another solution, setting ambient factor and use that hardcoded in the lighting pass is a quick hack, should not be done
		out_color[1] = vec4(diffuseColor.rgb,diffuseCoefficient);
		out_color[2] = vec4(specularColor.rgb,specularCoefficient);
		out_color[3] = vec4(vnormalBumped,specularHardness);
	}
}